{ INIT.INC }

{ *************************************************************************** }
{ *                                                                         * }
{ *                TURBO SCREEN INPUT PRE-PROCESSOR TOOLKIT                 * }
{ *                                                                         * }
{ *                 INITIALIZATION SUBPROGRAM INCLUDE FILE                  * }
{ *                                                                         * }
{ *                             Version  1.07                               * }
{ *                                                                         * }
{ *************************************************************************** }



Procedure InitModule;

{ *************************************************************************** }
{ *                                                                         * }
{ *                          INITIALIZATION MODULE                          * }
{ *                                                                         * }
{ *    This module controls the initialization of the input pre-processor   * }
{ *    program.  It reads all of the prompt files, initializes all of the   * }
{ *    data arrays, and sets-up the proper screen colors depending on       * }
{ *    the type of monitor being used.                                      * }
{ *                                                                         * }
{ *************************************************************************** }



  Procedure InitVariables;

  { This procedure initializes the global variables that are used in this
    program. }

  Begin { InitVariables }

    { variable for the specific example application of the input pre-processor }
    NewFile:=False;                    { set boolean flag for generating new input file to False }

    CurrentPage:=Menu;                 { set current page type module pointer to point to menu module }
    MenuPage:=1;                       { initialize the page numbers }
    G_I_Page:=1;
    S_I_Page:=1;
  End;  { InitVariables }



  Procedure InitScreenColors;

  { This procedure first determines what type of monitor is being used.  It
    then sets the proper screen colors for that monitor. }

  Begin   { InitScreenColors }
    If MonitorType=3 Then { color adapter and color monitor }
      Begin { Color Screen Constants }
        HighlightColor:=15;            { Highlight prompt }
        BackgroundColor:=1;            { Normal background color }
        ForegroundColor:=7;            { Normal foreground color }
        CharacterInputColor:=31;       { Color of characters being inputed }
        BlockBackgroundColor:=7;       { Color of highlighted input block background }
        BlockForegroundColor:=1;       { Color of highlighted input block foreground }
        QuickInputHighlightColor:=14;  { Color of special 1 or 2 character input prompts }
        InputWindowColor:=4;           { Color of load input box background }
        ErrorWindowColor:=1;           { Color of error message box background }
        InputWindowBorderColor:=3;     { Color of special input box border color }
        MenuBackgroundColor:=0;        { Color of main menu screen background }
      End { If MonitorType }
    Else { monochrome adapter or color adapter and monochrome monitor }
      Begin { Mono Screen Constants }
        HighlightColor:=15;            { Highlight prompt }
        BackgroundColor:=0;            { Normal background color }
        ForegroundColor:=7;            { Normal foreground color }
        CharacterInputColor:=31;       { Color of characters being inputed }
        BlockBackgroundColor:=7;       { Color of highlighted input block background }
        BlockForegroundColor:=0;       { Color of highlighted input block foreground }
        QuickInputHighlightColor:=15;  { Color of special 1 or 2 character input prompts }
        InputWindowColor:=0;           { Color of load input box background }
        ErrorWindowColor:=0;           { Color of error message box background }
        InputWindowBorderColor:=7;     { Color of special input box border color }
        MenuBackgroundColor:=0;        { Color of main menu screen background }
      End; { If MonitorType }
  End;    { InitScreenColors }



Begin   { InitModule }
  ReadMenuFiles;
  Read_G_I_Files;
  Init_G_I_DataEntries;
  Read_S_I_Files;
  Init_S_I_DataEntries;
  InitVariables;
  InitScreenColors;
  InitTextScreenPointers;
  HideBlinkingCursor;
End;    { InitModule }
